<?php
/*
Small script that queries info from a server and outputs this as an xml file.
This xml file can be parsed to output the info to for example an webpage.

Auther: Rolf Koedam (alias kE)
Based on: UT Query PHP script v1.2 by Almar Joling, 2003
*/

// Helper function
function GetMinutes($seconds) {
	$timemins = intval($seconds / 60);
	$timesecs = ($seconds % 60);

	$Reqlength = 2; //Amount of digits we need
	if ($Reqlength-strlen($timemins) > 0) $timemins = str_repeat("0",($Reqlength-strlen($timemins))) . $timemins;
	if ($Reqlength-strlen($timesecs) > 0) $timesecs = str_repeat("0",($Reqlength-strlen($timesecs))) . $timesecs;
	return $timemins . ":" . $timesecs;
}

$G_serverip = "xxx.xxx.xxx.xxx:xxxx";

// Get server IP info
list($serverip, $serverport) = split(":", $G_serverip);
$queryport = $serverport+1;

//If there is no IP, return (exit)
if (strlen($serverip) == 0) {
	echo "error1!";
	return;
}

//Open UDP socket to server
$sock = fsockopen("udp://" . $serverip, $queryport, $errno, $errstr,4);

//Check if we have a socket open, if not, display error message
if (!$sock) {
	echo "$errstr ($errno)<br>\n";
	exit;
}

fputs($sock,"\\status\\\game_property\ElapsedTime\\\game_property\RemainingTime\\\\teams\\\player_property\Health\\");

$gotfinal = False;
$data = "";

//Set starttime, for possible loop expiration, so the server doesn't get too much work.
$starttime = Time();

//Loop until final packet has been received.
while(!($gotfinal == True || feof($sock))) {
	//Get data
	if(($buf = fgetc($sock)) == FALSE) {
		usleep(100); // wait for additional data? :S whatever
	}
	//Add to databuffer
	$data .= $buf;
	//Check if final item (queryid) has been received
	if (strpos($data,"final\\") != False) {
		$gotfinal = True;
	}
	//Protect webserver against massive loop.
	if ((Time() - $starttime) > 5) {
		echo "Data receiving took too long. Cancelled.<P>";
		$gotfinal = True;
	}
}

//Close socket
fclose ($sock);


//Split chunks by \
$chunks = split('[\]', $data);

$pid = 0;
$playerid = "player_".$pid;
$tid = 0;
$teamid = "team_".$tid;
$i = 1;
$s = 0;
// Load first bit of server info
while($i <= 62) {
	$serverinfo[$s] = $chunks[$i];
	$s++;
	$i++;
}
// Keep loading player info if avail
while($chunks[$i] == $playerid || $chunks[$i] == "queryid") {
	if($chunks[$i] == "queryid") {
		$i=$i+2;		
	} else {
		$pi = 0;
		while($pi < 16) {
			$playerinfo[$pi] = $chunks[$i];
			$pi++;
			$i++;
		}
		$players[$pid] = $playerinfo;
		$pid++;
		$playerid = "player_".$pid;
	}
}
// Load next four lines of server info
for($n=0;$n<4;$n++) {
	$serverinfo[$s] = $chunks[$i];
	$s++;
	$i++;	
}
// Load team info if avail
while($chunks[$i] == $teamid || $chunks[$i] == "queryid") {
	if($chunks[$i] == "queryid") {
		$i=$i+2;		
	} else {
		$ti = 0;
		while($ti < 6) {
			$teaminfo[$ti] = $chunks[$i];
			$ti++;
			$i++;
		}
		$teams[$tid] = $teaminfo;
		$tid++;
		$teamid = "team_".$tid;
	}
}

//Output xml wannabe
echo "<?xml version='1.0' standalone='yes'?>\n";

//Output all serverinfo
echo "<server>\n";
echo "<serverinfo>\n";
for($i=0;$i<count($serverinfo);$i++) {
	$tag = htmlspecialchars(strtolower($serverinfo[$i]));
	if($tag == "elapsedtime") $serverinfo[$i+1] = GetMinutes($serverinfo[$i+1]);
	if($tag == "remainingtime") $serverinfo[$i+1] = GetMinutes($serverinfo[$i+1]);
	echo "<".$tag.">".htmlspecialchars($serverinfo[$i+1])."</".$tag.">\n";
	$i++;
}
echo "</serverinfo>\n";
//Output all playerinfo
echo "<players>\n";
for($p=0;$p<count($players);$p++) {
	$playerinfo = $players[$p];
	echo "<player>\n";
	for($i=0;$i<count($playerinfo);$i++) {
		$tag = strtolower(substr($playerinfo[$i],0,strlen($playerinfo[$i])-2));
		if($tag == "player") $tag = "nick";
		echo "<".$tag.">".htmlspecialchars($playerinfo[$i+1])."</".$tag.">\n";
		$i++;
	}
	echo "</player>\n";
}
echo "</players>\n";
//Output teaminfo
echo "<teams>\n";
for($t=0;$t<count($teams);$t++) {
	$teaminfo = $teams[$t];
	echo "<team>\n";
	for($i=0;$i<count($teaminfo);$i++) {
		$tag = strtolower(substr($teaminfo[$i],0,strlen($teaminfo[$i])-2));
		if($tag == "team") $tag = "color";
		echo "<".$tag.">".htmlspecialchars($teaminfo[$i+1])."</".$tag.">\n";
		$i++;
	}
	echo "</team>\n";
}
echo "</teams>\n";
echo "</server>\n";

?>